@@ -18,12 +18,12 @@ from threading import Thread
 from calibre import prints
 from calibre.constants import numeric_version, DEBUG, cache_dir
 from calibre.devices.errors import (OpenFailed, OpenFeedback, ControlError, TimeoutError,
-                                    InitialConnectionError, PacketError)
+                                    InitialConnectionError, PacketError, UserFeedback)
 from calibre.devices.interface import DevicePlugin, currently_connected_device
 from calibre.devices.usbms.books import Book, CollectionsBookList
 from calibre.devices.usbms.deviceconfig import DeviceConfig
 from calibre.devices.usbms.driver import USBMS
-from calibre.devices.utils import build_template_regexp
+from calibre.devices.utils import build_template_regexp, sanity_check
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.metadata import title_sort
 from calibre.ebooks.metadata.book.base import Metadata
@@ -252,6 +252,7 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
         'SET_LIBRARY_INFO'       : 19,
         'DELETE_BOOK'            : 13,
         'DISPLAY_MESSAGE'        : 17,
+        'ERROR'                  : 20,
         'FREE_SPACE'             : 5,
         'GET_BOOK_FILE_SEGMENT'  : 14,
         'GET_BOOK_METADATA'      : 15,
@@ -338,7 +339,11 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
               'to be able to send to the device. For example, you might have '
               'audio books in your library with the extension "m4b" that you '
               'want to listen to on your device. Don\'t worry about the "extra '
-              'enabled extensions" warning.')
+              'enabled extensions" warning.'),
+        _('Ignore device free space') + ':::<p>' +
+        _("Check this box to ignore the amount of free space reported by your "
+          "devices. This might be needed if you store books on an SD card and "
+          "the device doesn't have much free main memory.") + '</p>',
         ]
     EXTRA_CUSTOMIZATION_DEFAULT = [
                 False, '',
@@ -349,7 +354,7 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
                 False, '',
                 True,   '75',
                 True,   '',
-                ''
+                '',     False,
     ]
     OPT_AUTOSTART               = 0
     OPT_PASSWORD                = 2
@@ -363,6 +368,7 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
     OPT_COMPRESSION_QUALITY     = 13
     OPT_USE_METADATA_CACHE      = 14
     OPT_EXTRA_EXTENSIONS        = 16
+    OPT_IGNORE_FREESPACE        = 17
     OPTNAME_TO_NUMBER_MAP = {
         'password': OPT_PASSWORD,
         'autostart': OPT_AUTOSTART,
@@ -702,8 +708,12 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
                                'canSupportLpathChanges': True},
                           print_debug_info=False,
                           wait_for_response=self.can_send_ok_to_sendbook)
-
         if self.can_send_ok_to_sendbook:
+            if opcode == 'ERROR':
+                raise UserFeedback(msg='Sending book %s to device failed' % lpath,
+                                   details=result.get('message', ''),
+                                   level=UserFeedback.ERROR)
+                return
             lpath = result.get('lpath', lpath)
             book_metadata.lpath = lpath
         self._set_known_metadata(book_metadata)
@@ -1266,6 +1276,7 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
         self._debug()
         opcode, result = self._call_client('FREE_SPACE', {})
         if opcode == 'OK':
+            self._debug('free space:', result['free_space_on_device'])
             return (result['free_space_on_device'], 0, 0)
         # protocol error if we get here
         return (0, 0, 0)
@@ -1451,6 +1462,7 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
     @synchronous('sync_lock')
     def eject(self):
         self._debug()
+        self._call_client('NOOP', {'ejecting': True})
         self._close_device_socket()
 
     @synchronous('sync_lock')
@@ -1464,7 +1476,9 @@ class SMART_DEVICE_APP(DeviceConfig, DevicePlugin):
             self._debug(names)
         else:
             self._debug()
-
+        if not self.settings().extra_customization[self.OPT_IGNORE_FREESPACE]:
+            sanity_check(on_card='', files=files, card_prefixes=[],
+                         free_space=self.free_space())
         paths = []
         names = iter(names)
         metadata = iter(metadata)
