@@ -5,25 +5,29 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 from PyQt5.Qt import (Qt, QDialog, QTableWidgetItem, QIcon, QByteArray, QSize,
                       QDialogButtonBox, QTableWidget, QItemDelegate, QApplication,
-                      pyqtSignal)
+                      pyqtSignal, QAction, QFrame, QLabel, QTimer)
 
 from calibre.gui2.dialogs.tag_list_editor_ui import Ui_TagListEditor
 from calibre.gui2.complete2 import EditWithComplete
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.widgets import EnLineEdit
 from calibre.gui2 import question_dialog, error_dialog, gprefs
-from calibre.utils.icu import sort_key
+from calibre.utils.config import prefs
+from calibre.utils.icu import contains, primary_contains, primary_startswith
 from polyglot.builtins import unicode_type
 
+QT_HIDDEN_CLEAR_ACTION = '_q_qlineeditclearaction'
+
 
 class NameTableWidgetItem(QTableWidgetItem):
 
-    def __init__(self):
+    def __init__(self, sort_key):
         QTableWidgetItem.__init__(self)
         self.initial_value = ''
         self.current_value = ''
         self.is_deleted = False
         self.is_placeholder = False
+        self.sort_key = sort_key
 
     def data(self, role):
         if role == Qt.DisplayRole:
@@ -76,10 +80,12 @@ class NameTableWidgetItem(QTableWidgetItem):
             self.setText(self.text_before_placeholder)
 
     def __ge__(self, other):
-        return sort_key(unicode_type(self.text())) >= sort_key(unicode_type(other.text()))
+        return (self.sort_key(unicode_type(self.text())) >=
+                    self.sort_key(unicode_type(other.text())))
 
     def __lt__(self, other):
-        return sort_key(unicode_type(self.text())) < sort_key(unicode_type(other.text()))
+        return (self.sort_key(unicode_type(self.text())) <
+                    self.sort_key(unicode_type(other.text())))
 
 
 class CountTableWidgetItem(QTableWidgetItem):
@@ -129,14 +135,17 @@ class EditColumnDelegate(QItemDelegate):
 
 class TagListEditor(QDialog, Ui_TagListEditor):
 
-    def __init__(self, window, cat_name, tag_to_match, get_book_ids, sorter):
+    def __init__(self, window, cat_name, tag_to_match, get_book_ids, sorter,
+                 ttm_is_first_letter=False):
         QDialog.__init__(self, window)
         Ui_TagListEditor.__init__(self)
         self.setupUi(self)
+        self.verticalLayout_2.setAlignment(Qt.AlignCenter)
         self.search_box.setMinimumContentsLength(25)
 
         # Put the category name into the title bar
         t = self.windowTitle()
+        self.category_name = cat_name
         self.setWindowTitle(t + ' (' + cat_name + ')')
         # Remove help icon on title bar
         icon = self.windowIcon()
@@ -161,16 +170,14 @@ class TagListEditor(QDialog, Ui_TagListEditor):
         self.get_book_ids = get_book_ids
         self.text_before_editing = ''
 
-        # Set up the column headings
-        self.down_arrow_icon = QIcon(I('arrow-down.png'))
-        self.up_arrow_icon = QIcon(I('arrow-up.png'))
-        self.blank_icon = QIcon(I('blank.png'))
-
         # Capture clicks on the horizontal header to sort the table columns
         hh = self.table.horizontalHeader()
-        hh.setSectionsClickable(True)
-        hh.sectionClicked.connect(self.header_clicked)
         hh.sectionResized.connect(self.table_column_resized)
+        hh.setSectionsClickable(True)
+        hh.sectionClicked.connect(self.do_sort)
+        hh.setSortIndicatorShown(True)
+
+        self.last_sorted_by = 'name'
         self.name_order = 0
         self.count_order = 1
         self.was_order = 1
@@ -180,14 +187,13 @@ class TagListEditor(QDialog, Ui_TagListEditor):
         self.edit_delegate.editing_started.connect(self.start_editing)
         self.table.setItemDelegateForColumn(0, self.edit_delegate)
 
-        # Add the data
-        select_item = self.fill_in_table(None, tag_to_match)
-
-        # Scroll to the selected item if there is one
-        if select_item is not None:
-            self.table.setCurrentItem(select_item)
+        if prefs['use_primary_find_in_search']:
+            self.string_contains = primary_contains
+        else:
+            self.string_contains = contains
 
         self.delete_button.clicked.connect(self.delete_tags)
+        self.table.delete_pressed.connect(self.delete_pressed)
         self.rename_button.clicked.connect(self.rename_tag)
         self.undo_button.clicked.connect(self.undo_edit)
         self.table.itemDoubleClicked.connect(self._rename_tag)
@@ -198,8 +204,34 @@ class TagListEditor(QDialog, Ui_TagListEditor):
         self.buttonBox.accepted.connect(self.accepted)
 
         self.search_box.initialize('tag_list_search_box_' + cat_name)
-        self.search_button.clicked.connect(self.all_matching_clicked)
+        le = self.search_box.lineEdit()
+        ac = le.findChild(QAction, QT_HIDDEN_CLEAR_ACTION)
+        if ac is not None:
+            ac.triggered.connect(self.clear_search)
+        self.search_box.textChanged.connect(self.search_text_changed)
+        self.search_button.clicked.connect(self.do_search)
         self.search_button.setDefault(True)
+        l = QLabel(self.table)
+        self.not_found_label = l
+        l.setFrameStyle(QFrame.StyledPanel)
+        l.setAutoFillBackground(True)
+        l.setText(_('No matches found'))
+        l.setAlignment(Qt.AlignVCenter)
+        l.resize(l.sizeHint())
+        l.move(10, 0)
+        l.setVisible(False)
+        self.not_found_label_timer = QTimer()
+        self.not_found_label_timer.setSingleShot(True)
+        self.not_found_label_timer.timeout.connect(
+                self.not_found_label_timer_event, type=Qt.QueuedConnection)
+
+        self.filter_box.initialize('tag_list_filter_box_' + cat_name)
+        le = self.filter_box.lineEdit()
+        ac = le.findChild(QAction, QT_HIDDEN_CLEAR_ACTION)
+        if ac is not None:
+            ac.triggered.connect(self.clear_filter)
+        le.returnPressed.connect(self.do_filter)
+        self.filter_button.clicked.connect(self.do_filter)
 
         self.apply_vl_checkbox.clicked.connect(self.vl_box_changed)
 
@@ -213,17 +245,46 @@ class TagListEditor(QDialog, Ui_TagListEditor):
                 self.resize(self.sizeHint()+QSize(150, 100))
         except:
             pass
+        # Add the data
+        self.search_item_row = -1
+        self.fill_in_table(None, tag_to_match, ttm_is_first_letter)
 
     def vl_box_changed(self):
-        self.fill_in_table(None, None)
+        self.search_item_row = -1
+        self.fill_in_table(None, None, False)
 
-    def fill_in_table(self, tags, tag_to_match):
+    def do_search(self):
+        self.not_found_label.setVisible(False)
+        find_text = icu_lower(unicode_type(self.search_box.currentText()))
+        if not find_text:
+            return
+        for _ in range(0, self.table.rowCount()):
+            r = self.search_item_row = (self.search_item_row + 1) % self.table.rowCount()
+            if self.string_contains(find_text,
+                        self.table.item(r, 0).text()):
+                self.table.setCurrentItem(self.table.item(r, 0))
+                self.table.setFocus(True)
+                return
+        # Nothing found. Pop up the little dialog for 1.5 seconds
+        self.not_found_label.setVisible(True)
+        self.not_found_label_timer.start(1500)
+
+    def search_text_changed(self):
+        self.search_item_row = -1
+
+    def clear_search(self):
+        self.search_item_row = -1
+        self.search_box.setText('')
+
+    def fill_in_table(self, tags, tag_to_match, ttm_is_first_letter):
         data = self.get_book_ids(self.apply_vl_checkbox.isChecked())
         self.all_tags = {}
+        filter_text = icu_lower(unicode_type(self.filter_box.text()))
         for k,v,count in data:
-            self.all_tags[v] = {'key': k, 'count': count, 'cur_name': v,
-                                'is_deleted': k in self.to_delete}
-            self.original_names[k] = v
+            if not filter_text or self.string_contains(filter_text, icu_lower(v)):
+                self.all_tags[v] = {'key': k, 'count': count, 'cur_name': v,
+                                   'is_deleted': k in self.to_delete}
+                self.original_names[k] = v
         self.edit_delegate.set_completion_data(self.original_names.values())
 
         self.ordered_tags = sorted(self.all_tags.keys(), key=self.sorter)
@@ -234,20 +295,16 @@ class TagListEditor(QDialog, Ui_TagListEditor):
         self.table.blockSignals(True)
         self.table.clear()
         self.table.setColumnCount(3)
-        self.name_col = QTableWidgetItem(_('Tag'))
+        self.name_col = QTableWidgetItem(self.category_name)
         self.table.setHorizontalHeaderItem(0, self.name_col)
-        self.name_col.setIcon(self.up_arrow_icon)
         self.count_col = QTableWidgetItem(_('Count'))
         self.table.setHorizontalHeaderItem(1, self.count_col)
-        self.count_col.setIcon(self.blank_icon)
         self.was_col = QTableWidgetItem(_('Was'))
         self.table.setHorizontalHeaderItem(2, self.was_col)
-        self.count_col.setIcon(self.blank_icon)
 
         self.table.setRowCount(len(tags))
-
         for row,tag in enumerate(tags):
-            item = NameTableWidgetItem()
+            item = NameTableWidgetItem(self.sorter)
             item.set_is_deleted(self.all_tags[tag]['is_deleted'])
             _id = self.all_tags[tag]['key']
             item.setData(Qt.UserRole, _id)
@@ -258,9 +315,12 @@ class TagListEditor(QDialog, Ui_TagListEditor):
                 item.setText(tag)
             item.setFlags(item.flags() | Qt.ItemIsSelectable | Qt.ItemIsEditable)
             self.table.setItem(row, 0, item)
-            if tag == tag_to_match:
-                select_item = item
-
+            if select_item is None:
+                if ttm_is_first_letter:
+                    if primary_startswith(tag, tag_to_match):
+                        select_item = item
+                elif tag == tag_to_match:
+                    select_item = item
             item = CountTableWidgetItem(self.all_tags[tag]['count'])
             # only the name column can be selected
             item.setFlags(item.flags() & ~(Qt.ItemIsSelectable|Qt.ItemIsEditable))
@@ -271,27 +331,31 @@ class TagListEditor(QDialog, Ui_TagListEditor):
             if _id in self.to_rename or _id in self.to_delete:
                 item.setData(Qt.DisplayRole, tag)
             self.table.setItem(row, 2, item)
-        self.table.blockSignals(False)
-        return select_item
 
-    def all_matching_clicked(self):
-        for i in range(0, self.table.rowCount()):
-            item = self.table.item(i, 0)
-            tag = item.initial_text()
-            self.all_tags[tag]['cur_name'] = item.text()
-            self.all_tags[tag]['is_deleted'] = item.is_deleted
-        search_for = icu_lower(unicode_type(self.search_box.text()))
-        if len(search_for) == 0:
-            self.fill_in_table(None, None)
-        result = []
-        for k in self.ordered_tags:
-            tag = self.all_tags[k]
-            if (
-                search_for in icu_lower(unicode_type(tag['cur_name'])) or
-                search_for in icu_lower(unicode_type(self.original_names.get(tag['key'], '')))
-            ):
-                result.append(k)
-        self.fill_in_table(result, None)
+        if self.last_sorted_by == 'name':
+            self.table.sortByColumn(0, self.name_order)
+        elif self.last_sorted_by == 'count':
+            self.table.sortByColumn(1, self.count_order)
+        else:
+            self.table.sortByColumn(2, self.was_order)
+
+        if select_item is not None:
+            self.table.setCurrentItem(select_item)
+            self.start_find_pos = select_item.row()
+        else:
+            self.table.setCurrentCell(0, 0)
+            self.start_find_pos = -1
+        self.table.blockSignals(False)
+
+    def not_found_label_timer_event(self):
+        self.not_found_label.setVisible(False)
+
+    def clear_filter(self):
+        self.filter_box.setText('')
+        self.fill_in_table(None, None, False)
+
+    def do_filter(self):
+        self.fill_in_table(None, None, False)
 
     def table_column_resized(self, col, old, new):
         self.table_column_widths = []
@@ -397,6 +461,10 @@ class TagListEditor(QDialog, Ui_TagListEditor):
         else:
             self.table.editItem(item)
 
+    def delete_pressed(self):
+        if self.table.currentColumn() == 0:
+            self.delete_tags()
+
     def delete_tags(self):
         deletes = self.table.selectedItems()
         if not deletes:
@@ -405,77 +473,48 @@ class TagListEditor(QDialog, Ui_TagListEditor):
             return
 
         to_del = []
-        to_undel = []
         for item in deletes:
-            if item.is_deleted:
-                to_undel.append(item)
-            else:
+            if not item.is_deleted:
                 to_del.append(item)
+
         if to_del:
             ct = ', '.join([unicode_type(item.text()) for item in to_del])
             if not confirm(
                 '<p>'+_('Are you sure you want to delete the following items?')+'<br>'+ct,
                 'tag_list_editor_delete'):
                 return
-        if to_undel:
-            ct = ', '.join([unicode_type(item.text()) for item in to_undel])
-            if not confirm(
-                '<p>'+_('Are you sure you want to undelete the following items?')+'<br>'+ct,
-                'tag_list_editor_undelete'):
-                return
+
         row = self.table.row(deletes[0])
+        self.table.blockSignals(True)
         for item in deletes:
-            if item.is_deleted:
-                item.set_is_deleted(False)
-                self.to_delete.discard(int(item.data(Qt.UserRole)))
-                orig = self.table.item(item.row(), 2)
-                self.table.blockSignals(True)
-                orig.setData(Qt.DisplayRole, '')
-                self.table.blockSignals(False)
-            else:
-                id = int(item.data(Qt.UserRole))
-                self.to_delete.add(id)
-                item.set_is_deleted(True)
-                orig = self.table.item(item.row(), 2)
-                self.table.blockSignals(True)
-                orig.setData(Qt.DisplayRole, item.initial_text())
-                self.table.blockSignals(False)
+            id_ = int(item.data(Qt.UserRole))
+            self.to_delete.add(id_)
+            item.set_is_deleted(True)
+            orig = self.table.item(item.row(), 2)
+            orig.setData(Qt.DisplayRole, item.initial_text())
+        self.table.blockSignals(False)
         if row >= self.table.rowCount():
             row = self.table.rowCount() - 1
         if row >= 0:
             self.table.scrollToItem(self.table.item(row, 0))
 
-    def header_clicked(self, idx):
-        if idx == 0:
-            self.do_sort_by_name()
-        elif idx == 1:
-            self.do_sort_by_count()
-        else:
-            self.do_sort_by_was()
+    def do_sort(self, section):
+        (self.do_sort_by_name, self.do_sort_by_count, self.do_sort_by_was)[section]()
 
     def do_sort_by_name(self):
-        self.name_order = 1 if self.name_order == 0 else 0
+        self.name_order = 1 - self.name_order
+        self.last_sorted_by = 'name'
         self.table.sortByColumn(0, self.name_order)
-        self.name_col.setIcon(self.down_arrow_icon if self.name_order
-                                                    else self.up_arrow_icon)
-        self.count_col.setIcon(self.blank_icon)
-        self.was_col.setIcon(self.blank_icon)
 
     def do_sort_by_count(self):
-        self.count_order = 1 if self.count_order == 0 else 0
+        self.count_order = 1 - self.count_order
+        self.last_sorted_by = 'count'
         self.table.sortByColumn(1, self.count_order)
-        self.count_col.setIcon(self.down_arrow_icon if self.count_order
-                                                    else self.up_arrow_icon)
-        self.name_col.setIcon(self.blank_icon)
-        self.was_col.setIcon(self.blank_icon)
 
     def do_sort_by_was(self):
-        self.was_order = 1 if self.was_order == 0 else 0
+        self.was_order = 1 - self.was_order
+        self.last_sorted_by = 'count'
         self.table.sortByColumn(2, self.was_order)
-        self.was_col.setIcon(self.down_arrow_icon if self.was_order
-                                                    else self.up_arrow_icon)
-        self.name_col.setIcon(self.blank_icon)
-        self.count_col.setIcon(self.blank_icon)
 
     def accepted(self):
         self.save_geometry()
